module Visual (build, Image(..)) where

import Data.Time (UTCTime)
import Data.List (isSuffixOf)
import qualified Config
import Common
import Templates (outerWith, loading_)
import Lucid
import Route


thumbWidth :: Int
thumbWidth = 710


data Image = Image
    { imgPath        :: FilePath
    , imgThumbPath   :: FilePath
    , imgThumbWidth  :: Int
    , imgThumbHeight :: Int
    , imgDate        :: UTCTime
    , imgNSFW        :: Bool     -- is the image nsfw?
    } deriving (Generic, Eq, Binary)


data EntryMeta = EntryMeta
    { title     :: Text
    , date      :: Text
    , updated   :: Maybe Text
    } deriving (Generic, Eq, FromJSON)


data Entry = Entry
    { entryTitle   :: Text
    , entryContent :: Text
    , entryItems   :: [Image]
    , entryDate    :: UTCTime
    , entryType    :: Text
    } deriving (Generic, Eq, Binary)


instance IsTimestamped Image where timestamp = imgDate
instance IsTimestamped Entry where timestamp = entryDate


build :: Task IO [Image]
build = do
    pictures <- match "visual/*" \src -> do
        path  <- copyFile src
        tpath <- callCommandWith (\a b -> "convert -resize 710x " <> a <> " " <> b)
                                 (-<.> "thumb.webp")
                                 src
        apath <- toAbsolute src
        size  <- read <$> readCommand "identify" ["-ping", "-format", "(%w, %h)", apath]
        let nsfw = "nsfw" `isSuffixOf` dropExtension src
        pure (Image path tpath thumbWidth (thumbHeight size) (timestamp src) nsfw)


    -- webcomics/albums
    entries <- matchDir "visual/*/" \src -> do
        matchFile "index.markdown" \src -> do
            (meta, doc) <- readPandocMetadata src
            renderPandoc doc
                <&> renderEntry meta
                >>= write (src -<.> "html")
        pure ()


    watch pictures do
        let sorted = filter (not . imgNSFW) $ recentFirst pictures

        match_ "./visual.rst" \src -> do
            intro <- compilePandoc src
            write "visual.html" $ renderVisual intro sorted

        return (take 4 sorted)
    where
        thumbHeight :: (Int, Int) -> Int
        thumbHeight (width, height) = round (fi height * fi thumbWidth / fi width)

        fi :: Int -> Float
        fi = fromIntegral


renderEntry :: EntryMeta -> Text -> Html ()
renderEntry meta content =
    outerWith def { Config.title = title meta
                  , Config.route = VEntryRoute
                  } do
        header_ do
            h1_ $ toHtml (title meta)
            p_ $ toHtml (date meta)
        toHtmlRaw content
        hr_ []

renderVisual :: Text -> [Image] -> Html ()
renderVisual txt imgs =
    outerWith def {Config.title = "visual"} do
        toHtmlRaw txt
        hr_ []
        section_ [class_ "visual"] $
            forM_ imgs \Image{..} ->
                figure_ $ a_ [href_ $ fromString imgPath] $ img_
                    [ src_ (fromString imgThumbPath)
                    , width_ (fromString $ show imgThumbWidth)
                    , height_ (fromString $ show imgThumbHeight)
                    , loading_ "lazy" ]
